#ifndef HistoBook_H
#define HistoBook_H
/**
 * @file HistoBook.h
 * Description of cpp file
 *
 * @brief A brief description
 *
 * @author Jakub Cuth <Jakub.Cuth@cern.ch>
 * @date 2016-08-29
 */

#include "HistoHandler.h"
#include "KinematicDefinitions.h"
#include "settings.h"
#include "user_book.h"

using namespace Kinematics;
namespace HistoHandler{
    void Book() {
        DeleteHists();
        Add( new Histo1D        <BosPT           > ("qt"          ) );
	Add( new Histo1D        <BosY            > ("y"          ) );
	Add( new Histo1D        <BosM            > ("m"          ) );
        Add( new Histo1D        <LepPT           > ("ptl"          ) );
        Add( new Histo1D        <LepEta          > ("etal"          ) );
        Add( new Histo2D        <BosPT,BosY      > ("qt","y"      ) );
        Add( new Histo3D        <BosPT,BosY,BosM > ("qt","y","m"  ) );
        Add( new HistoProfile   <BosPT,A1        > ("qt","a1"     ) );
        Add( new HistoProfile2D <BosPT,BosY,A1   > ("qt","y","a1" ) );

	// book a set of optional default histograms
	if (opts.hetal)
	  if (opts.nproc == 1)
	    Add( new Histo1D        <ALpAbsEta          > ("etalep"          ) )->SetName("eta_lep");
	  else if (opts.nproc == 2)
	    Add( new Histo1D        <LepAbsEta          > ("etalep"          ) )->SetName("eta_lep");

	if (opts.hptl)
	  if (opts.nproc == 1)
	    Add( new Histo1D        <ALpPT           > ("ptlep"          ) )->SetName("pt_lep");
	  else if (opts.nproc == 2)
	    Add( new Histo1D        <LepPT           > ("ptlep"          ) )->SetName("pt_lep");

	if (opts.hptnu)
	  if (opts.nproc == 1)
	    Add( new Histo1D        <LepPT           > ("ptnu"          ) )->SetName("pt_nu");
	  else if (opts.nproc == 2)
	    Add( new Histo1D        <ALpPT           > ("ptnu"          ) )->SetName("pt_nu");
	
	if (opts.hmt)
	  Add( new Histo1D        <BosMT           > ("mt"          ) )->SetName("mt");

	//if (opts.hphistareta) //to be implemented
	//Add( new Histo1D        <BosPhist           > ("phistareta"          ) )->SetName("phistareta");
	
        // book user histograms
        UserBook();
    }
}

#endif /* ifndef HistoBook_H */
